import functools
import typing as t

from flask import current_app, has_app_context

__all__ = ["template_global"]

g_template_global = {}


def template_global(name: str = None) -> t.Callable:
    fn = None
    if callable(name) and hasattr(name, "__name__"):
        fn = name
        name = fn.__name__

    def add_template_global(fn, name):
        if has_app_context():
            if name in g_template_global:
                return

            # This will fail with the flask error
            app = current_app.flasket
            app.logger.warning(f"Adding template global '{name}' is no longer possible during runtime.")
        else:
            g_template_global[name] = fn

    @functools.wraps(fn)
    def wrapped(fn):
        nonlocal name

        add_template_global(fn, name)
        return fn

    if fn:
        add_template_global(fn, name)
    return wrapped



# FIXME: get rid of this file?
import os
from copy import deepcopy

from boltons.iterutils import remap
from torxtools.dicttools import _deepmerge, strip_none


## TODO: Use torxtools.deepmerge
def deepmerge(dest, src):
    """
    Take every key, value from src and merge it recursivly into dest.
    None values are stripped from src before merging.

    Adapted from https://stackoverflow.com/questions/7204805

    Parameters
    ----------
    dest: dict
        destination dictionary

    src: dict
        source dictionary

    Returns
    -------
    dict:
        merged dictionary
    """
    # pylint: disable=unnecessary-lambda-assignment
    src = remap(src, visit=strip_none)

    return _deepmerge(src, deepcopy(dest))


def get_module_name(filepath: str, syspath: str) -> str:
    filepath = os.path.relpath(filepath, syspath)
    return filepath.replace("/", ".").replace(".py", "")
